## imports are included from imports.py ##

from PyQt5 import QtCore, QtGui, QtWidgets, QtWebEngineWidgets
from . import cyclograph_rc

def cg_translate(context, text, **kwargs):
    return _(text)

## end of imports ##
